﻿var map;
var baseLayer = null;
var layers = [];
var rasterLayer;
var icons = {
    0: "./icon/dot.png",
    1: "./icon/dot.png",
    2: "./icon/dot.png",
    3: "./icon/dot.png"
};
var LayerType = [];
LayerType.FluctSuihei = 0;
LayerType.FluctHyoko = 1;

function initialize() {
    map = createMap('http://cyberjapandata.gsi.go.jp/xyz/std/{z}/{x}/{y}.png', 35.3622222, 138.7313889, 5);
}

function createMap(url, lat, lon, zoom) {
    var newMap = L.map('map', {
        minZoom: zoom,
        maxBounds: [[50, 110], [7, 157]]
    });

    baseLayer = createBaseLayer(url);
    baseLayer.addTo(newMap);

    newMap.setView([lat, lon], zoom);

    newMap.on('moveend', onLocationEnd);
    newMap.on('click', onClick);
    newMap.on('mousemove', onMouseMove);

    L.control.scale({
        imperial: false
    }).addTo(newMap);

    newMap.attributionControl.setPrefix(null);

    return newMap;
}

function createBaseLayer(url) {
    layer = L.tileLayer(url, {
        tileSize: 256,
        attribution: "<a href='http://maps.gsi.go.jp/development/ichiran.html' target='_blank'>地理院タイル</a>"
    });

    return layer;
}

function onLocationEnd() {
    // fromMap.js
    sendLocation();
}

function onClick(e) {
    // fromMap.js
    onClickFeature(e);
}

function onMouseMove(e) {
    // fromMap.js
    mouseMove(e);
}

function getLayer(layerKey) {
    return layers[layerKey].layer;
}

function addLayer(layerKey) {
    var newLayer = L.featureGroup();
    newLayer.layerKey = layerKey;
    newLayer.addTo(map);

    layers[layerKey] = {
        layerID: newLayer._leaflet_id,
        layer: newLayer,
        visible: true,
        features: []
    }

    return newLayer;
}

function removeLayer(layerKey) {
    map.removeLayer(getLayer(layerKey));
    delete layers[layerKey];
}

function getFeature(layerKey, featureKey) {
    return layers[layerKey].features[featureKey].feature;
}

function addFeature(layerKey, layerType, featureKey, lat, lon, data) {
    //function addFeature(layerKey, featureKey, lat, lon) {

    var feature;

    switch (layerType) {
        case LayerType.FluctSuihei:
        case LayerType.FluctHyoko:
            feature = createFluctFeature(layerKey, layerType, featureKey, lat, lon, data);
            break;

        default:
            alert("unknownType:" + layerType);
            break;
    }

    layers[layerKey].features[featureKey] = {
        "feature": feature,
        "featureID": feature._leaflet_id
    };

    return feature;
}

function createFluctFeature(layerKey, layerType, featureKey, lat, lon, data) {

    var url = icons[data.icon];

    var feature = new L.marker([lat, lon], { icon: createIcon(url) });
    feature.layerKey = layerKey;
    feature.layerType = layerType;
    feature.featureKey = featureKey;
    feature.data = data;
    //feature.bindLabel("" + data.key.substring(0, 4) + "-" + data.key.substring(4, 8), {
    //    noHide: true,
    //    className: "iconlabel_" + labelType,
    //    direction: "right",
    //});

    feature.addTo(getLayer(layerKey));

    feature.on('click', onClickFeature);
    //feature.on('dragend', onDragEndFeature);

    return feature;
}

function removeFeature(layerKey, featureKey) {
    getLayer(layerKey).removeLayer(getFeature(layerKey, featureKey));
    delete layers[layerKey].features[featureKey];
}

function removeAllFeatures(layerKey) {
    getLayer(layerKey).clearLayers();
    layers[layerKey].features = [];
}

function createIcon(url) {

    var featureIcon = L.icon({
        iconUrl: url,
        iconSize: [16, 16]
    });

    return featureIcon;
}

function isExistLayer(layerKey) {
    return layers[layerKey] != null;
}

function isExistFeature(layerKey, featureKey) {
    return (layers[layerKey] != null && layers[layerKey].features[featureKey] != null);
}

// アイコンのパスを設定する
// in: オブジェクト（JSON文字列）
// {"icons":[{"type":type1,"path":path1},{"type":type2,"path":path2},…]}
function setIcons(iconStr) {

    var datas = JSON.parse(iconStr);
    for (var i = 0; i < datas.icons.length; i++) {
        data = datas.icons[i];
        icons[data.type] = data.path;
    }
}

// 国土地理院の標高APIを用いて、与えられた緯度経度から標高を取得する
function getAlt(lat, lon) {
    var xmlHttpRequest = new XMLHttpRequest();
    var alt = null;
    var errorMessage = "標高を取得できなかったため、空欄としました。標高が必要な場合は、時間を置いて再実行してみてください。";

    // HTTP通信の状態が変化したときのイベント
    xmlHttpRequest.onreadystatechange = function () {
        if (xmlHttpRequest.readyState === 4 && xmlHttpRequest.status === 200) {
            // 通信に成功した場合
            var elevationJsonData = eval("(" + xmlHttpRequest.responseText + ")");
            alt = elevationJsonData.elevation;
        }
    };

    // サーバからレスポンスされたエラーの処理
    xmlHttpRequest.onload = function () {
        if (xmlHttpRequest.readyState >= 2 && xmlHttpRequest.status !== 200) {
            alert(errorMessage);
        }
    }

    var getElevationApiUrl = "http://cyberjapandata2.gsi.go.jp/general/dem/scripts/getelevation.php?lat=" + lat + "&lon=" + lon + "&outtype=JSON";
    try {
        xmlHttpRequest.open("GET", getElevationApiUrl, false);
        xmlHttpRequest.timeout = 5000;
        xmlHttpRequest.send(null);
    } catch (e) {
        // 同期通信の場合、ネットワークエラーやタイムアウトエラーは例外として投げられるのでキャッチ
        alert(errorMessage);
    }

    return alt;
}
